/*
 * Copyright (c) 2005 (Mike) Maurice Kienenberger (mkienenb@alaska.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package org.gamenet.application.mm8leveleditor.data.mm6.fileFormat;

import java.util.ArrayList;
import java.util.List;

import org.gamenet.swing.controls.ComparativeTableControl;
import org.gamenet.util.ByteConversions;

public class DPftBin
{
    // Player Frame Table
    private static final int DPFT_BIN_RECORD_SIZE = 10;
    
    private int dPftBinOffset = 0;
    private byte dPftBinData[] = null;

    public DPftBin()
    {
        super();
    }

    public DPftBin(String fileName)
    {
        super();
        this.dPftBinData = new byte[DPFT_BIN_RECORD_SIZE];
    }

    public int initialize(byte dataSrc[], int offset)
    {
        this.dPftBinOffset = offset;
        this.dPftBinData = new byte[DPFT_BIN_RECORD_SIZE];
        System.arraycopy(dataSrc, offset, this.dPftBinData, 0, this.dPftBinData.length);
        offset += this.dPftBinData.length;
        
        return offset;
    }

    public static boolean checkDataIntegrity(byte[] data, int offset, int expectedNewOffset)
    {
        int count = ByteConversions.getIntegerInByteArrayAtPosition(data, offset);
        offset += 4;

        offset += count * DPFT_BIN_RECORD_SIZE;
        
        return (offset == expectedNewOffset);
    }
    
    public static int populateObjects(byte[] data, int offset, List dPftBinList)
    {
        int dPftBinCount = ByteConversions.getIntegerInByteArrayAtPosition(data, offset);
        offset += 4;

        for (int dPftBinIndex = 0; dPftBinIndex < dPftBinCount; ++dPftBinIndex)
        {
            DPftBin dPftBin = new DPftBin();
            dPftBinList.add(dPftBin);
            offset = dPftBin.initialize(data, offset);
        }
        
        return offset;
    }
    
    public static int updateData(byte[] newData, int offset, List dPftBinList)
    {
        ByteConversions.setIntegerInByteArrayAtPosition(dPftBinList.size(), newData, offset);
        offset += 4;

        for (int dPftBinIndex = 0; dPftBinIndex < dPftBinList.size(); ++dPftBinIndex)
        {
            DPftBin dPftBin = (DPftBin)dPftBinList.get(dPftBinIndex);
            System.arraycopy(dPftBin.getDPftBinData(), 0, newData, offset, dPftBin.getDPftBinData().length);
            offset += dPftBin.getDPftBinData().length;
        }
        
        return offset;
    }

    public byte[] getDPftBinData()
    {
        return this.dPftBinData;
    }
    public int getDPftBinOffset()
    {
        return this.dPftBinOffset;
    }

    public static int getRecordSize()
    {
        return DPFT_BIN_RECORD_SIZE;
    }


    // DPftBin things to decode

    public static List getOffsetList()
    {
        List offsetList = new ArrayList();
        offsetList.add(new ComparativeTableControl.OffsetData(0, DPFT_BIN_RECORD_SIZE, ComparativeTableControl.REPRESENTATION_SHORT_DEC));

        return offsetList;
    }
    
    public static ComparativeTableControl.DataSource getComparativeDataSource(final List dPftBinList)
    {
        return new ComparativeTableControl.DataSource()
        {
            public int getRowCount()
            {
                return dPftBinList.size();
            }

            public byte[] getData(int dataRow)
            {
                DPftBin dPftBin = (DPftBin)dPftBinList.get(dataRow);
                return dPftBin.getDPftBinData();
            }

            public int getAdjustedDataRowIndex(int dataRow)
            {
                return dataRow;
            }
            
            public String getIdentifier(int dataRow)
            {
                return "";
            }

            public int getOffset(int dataRow)
            {
                return 0;
            }
        };
    }
}
