/*
 * Copyright (c) 2005 (Mike) Maurice Kienenberger (mkienenb@alaska.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package org.gamenet.application.mm8leveleditor.data.mm7.fileFormat;

import java.util.ArrayList;
import java.util.List;

import org.gamenet.application.mm8leveleditor.data.DObjListBin;
import org.gamenet.swing.controls.ComparativeTableControl;
import org.gamenet.util.ByteConversions;

public class DObjListBinMM7 implements DObjListBin
{
    //  Name	Label	Type	Radius	Height	Lifetime	Speed	Attributes -

    private static final int DOBJLIST_BIN_RECORD_SIZE = 56;
    
    private int dObjListBinOffset = 0;
    private byte dObjListBinData[] = null;

    public DObjListBinMM7()
    {
        super();
    }

    public DObjListBinMM7(String fileName)
    {
        super();
        this.dObjListBinData = new byte[DOBJLIST_BIN_RECORD_SIZE];
    }

    public int initialize(byte dataSrc[], int offset)
    {
        this.dObjListBinOffset = offset;
        this.dObjListBinData = new byte[DOBJLIST_BIN_RECORD_SIZE];
        System.arraycopy(dataSrc, offset, this.dObjListBinData, 0, this.dObjListBinData.length);
        offset += this.dObjListBinData.length;
        
        return offset;
    }

    public static boolean checkDataIntegrity(byte[] data, int offset, int expectedNewOffset)
    {
        int count = ByteConversions.getIntegerInByteArrayAtPosition(data, offset);
        offset += 4;

        offset += count * DOBJLIST_BIN_RECORD_SIZE;
        
        return (offset == expectedNewOffset);
    }
    
    public static int populateObjects(byte[] data, int offset, List dObjListBinList)
    {
        int dObjListBinCount = ByteConversions.getIntegerInByteArrayAtPosition(data, offset);
        offset += 4;

        for (int dObjListBinIndex = 0; dObjListBinIndex < dObjListBinCount; ++dObjListBinIndex)
        {
            DObjListBinMM7 dObjListBin = new DObjListBinMM7();
            dObjListBinList.add(dObjListBin);
            offset = dObjListBin.initialize(data, offset);
        }
        
        return offset;
    }
    
    public static int updateData(byte[] newData, int offset, List dObjListBinList)
    {
        ByteConversions.setIntegerInByteArrayAtPosition(dObjListBinList.size(), newData, offset);
        offset += 4;

        for (int dObjListBinIndex = 0; dObjListBinIndex < dObjListBinList.size(); ++dObjListBinIndex)
        {
            DObjListBinMM7 dObjListBin = (DObjListBinMM7)dObjListBinList.get(dObjListBinIndex);
            System.arraycopy(dObjListBin.getDObjListBinData(), 0, newData, offset, dObjListBin.getDObjListBinData().length);
            offset += dObjListBin.getDObjListBinData().length;
        }
        
        return offset;
    }

    public byte[] getDObjListBinData()
    {
        return this.dObjListBinData;
    }
    public int getDObjListBinOffset()
    {
        return this.dObjListBinOffset;
    }

    public static int getRecordSize()
    {
        return DOBJLIST_BIN_RECORD_SIZE;
    }


    // DObjListBinMM7 things to decode

    public static List getOffsetList()
    {
        List offsetList = new ArrayList();
        offsetList.add(new ComparativeTableControl.OffsetData(0, 32, ComparativeTableControl.REPRESENTATION_STRING, "Name"));
        offsetList.add(new ComparativeTableControl.OffsetData(32, 2, ComparativeTableControl.REPRESENTATION_SHORT_DEC, "type?"));
        offsetList.add(new ComparativeTableControl.OffsetData(34, 2, ComparativeTableControl.REPRESENTATION_SHORT_DEC, "radius?"));
        offsetList.add(new ComparativeTableControl.OffsetData(36, 2, ComparativeTableControl.REPRESENTATION_SHORT_DEC, "height?"));
        offsetList.add(new ComparativeTableControl.OffsetData(38, 2, ComparativeTableControl.REPRESENTATION_SHORT_DEC, "attrib?"));
        offsetList.add(new ComparativeTableControl.OffsetData(40, 2, ComparativeTableControl.REPRESENTATION_SHORT_DEC, "Dsft ID #"));
        offsetList.add(new ComparativeTableControl.OffsetData(42, 8, ComparativeTableControl.REPRESENTATION_BYTE_DEC, "colorbit?"));
        offsetList.add(new ComparativeTableControl.OffsetData(50, 2, ComparativeTableControl.REPRESENTATION_SHORT_DEC, "speed?"));
        offsetList.add(new ComparativeTableControl.OffsetData(52, 4, ComparativeTableControl.REPRESENTATION_BYTE_DEC, "color?"));

        return offsetList;
    }
    
    public static ComparativeTableControl.DataSource getComparativeDataSource(final List dObjListBinList)
    {
        return new ComparativeTableControl.DataSource()
        {
            public int getRowCount()
            {
                return dObjListBinList.size();
            }

            public byte[] getData(int dataRow)
            {
                DObjListBinMM7 dObjListBin = (DObjListBinMM7)dObjListBinList.get(dataRow);
                return dObjListBin.getDObjListBinData();
            }

            public int getAdjustedDataRowIndex(int dataRow)
            {
                return dataRow;
            }
            
            public String getIdentifier(int dataRow)
            {
                return "";
            }

            public int getOffset(int dataRow)
            {
                return 0;
            }
        };
    }
}
